@@ -2,7 +2,7 @@
 **
 ** This file is part of Qt Creator
 **
-** Copyright (c) 2009 Nokia Corporation and/or its subsidiary(-ies).
+** Copyright (c) 2010 Nokia Corporation and/or its subsidiary(-ies).
 **
 ** Contact: Nokia Corporation (qt-info@nokia.com)
 **
@@ -44,19 +44,18 @@
 #include <QtGui/QSplitter>
 #include <QtGui/QSyntaxHighlighter>
 #include <QtGui/QTextBlock>
-
-#ifndef GDBDEBUGGERLEAN
+#include <QtGui/QPlainTextEdit>
 
 #include <aggregation/aggregate.h>
 #include <coreplugin/findplaceholder.h>
+#include <coreplugin/minisplitter.h>
 #include <find/basetextfind.h>
 
-using namespace Find;
-
-#endif // GDBDEBUGGERLEAN
+#include <utils/savedaction.h>
 
 using namespace Debugger;
 using namespace Debugger::Internal;
+using namespace Find;
 
 static QChar charForChannel(int channel)
 {
@@ -87,6 +86,7 @@ static LogChannel channelForChar(QChar c)
     }
 }
 
+
 /////////////////////////////////////////////////////////////////////
 //
 // OutputHighlighter
@@ -137,9 +137,37 @@ private:
     QPlainTextEdit *m_parent;
 };
 
+
 /////////////////////////////////////////////////////////////////////
 //
-// InputPane
+// InputHighlighter
+//
+/////////////////////////////////////////////////////////////////////
+
+class InputHighlighter : public QSyntaxHighlighter
+{
+public:
+    InputHighlighter(QPlainTextEdit *parent)
+        : QSyntaxHighlighter(parent->document()), m_parent(parent)
+    {}
+
+private:
+    void highlightBlock(const QString &text)
+    {
+        if (text.size() > 3 && text.at(2) == QLatin1Char(':')) {
+            QTextCharFormat format;
+            format.setForeground(Qt::darkRed);
+            setFormat(1, text.size(), format);
+        }
+    }
+
+    QPlainTextEdit *m_parent;
+};
+
+
+/////////////////////////////////////////////////////////////////////
+//
+// DebbuggerPane base class
 //
 /////////////////////////////////////////////////////////////////////
 
@@ -152,14 +180,15 @@ public:
         : QPlainTextEdit(parent)
     {
         setMaximumBlockCount(100000);
+        setFrameStyle(QFrame::NoFrame);
         m_clearContentsAction = new QAction(this);
-        m_clearContentsAction->setText(tr("Clear contents"));
+        m_clearContentsAction->setText(tr("Clear Contents"));
         m_clearContentsAction->setEnabled(true);
         connect(m_clearContentsAction, SIGNAL(triggered(bool)),
             parent, SLOT(clearContents()));
 
         m_saveContentsAction = new QAction(this);
-        m_saveContentsAction->setText(tr("Save contents"));
+        m_saveContentsAction->setText(tr("Save Contents"));
         m_saveContentsAction->setEnabled(true);
     }
 
@@ -172,6 +201,7 @@ public:
         theDebuggerAction(ExecuteCommand)->setData(textCursor().block().text());
         menu->addAction(theDebuggerAction(ExecuteCommand));
         menu->addAction(theDebuggerAction(LogTimeStamps));
+        menu->addAction(theDebuggerAction(VerboseLog));
         menu->addSeparator();
         menu->addAction(theDebuggerAction(SettingsDialog));
         menu->exec(ev->globalPos());
@@ -185,13 +215,22 @@ public:
     QAction *m_saveContentsAction;
 };
 
+
+/////////////////////////////////////////////////////////////////////
+//
+// InputPane
+//
+/////////////////////////////////////////////////////////////////////
+
 class InputPane : public DebuggerPane
 {
     Q_OBJECT
 public:
     InputPane(QWidget *parent)
         : DebuggerPane(parent)
-    {}
+    {
+        (void) new InputHighlighter(this);
+    }
 
 signals:
     void clearContentsRequested();
@@ -267,7 +306,7 @@ public slots:
     void gotoResult(int i)
     {
         QString needle = QString::number(i) + '^';
-        QString needle2 = QLatin1String(">") + needle;
+        QString needle2 = QLatin1Char('>') + needle;
         QTextCursor cursor(document());
         do {
             const QString line = cursor.block().text();
@@ -295,7 +334,8 @@ DebuggerOutputWindow::DebuggerOutputWindow(QWidget *parent)
 {
     setWindowTitle(tr("Debugger"));
 
-    QSplitter *m_splitter = new QSplitter(Qt::Horizontal, this);
+    QSplitter *m_splitter = new  Core::MiniSplitter(Qt::Horizontal);
+    m_splitter->setParent(this);
     // mixed input/output
     m_combinedText = new CombinedPane(this);
     m_combinedText->setReadOnly(true);
@@ -316,12 +356,9 @@ DebuggerOutputWindow::DebuggerOutputWindow(QWidget *parent)
     layout->setMargin(0);
     layout->setSpacing(0);
     layout->addWidget(m_splitter);
-#ifndef GDBDEBUGGERLEAN
     layout->addWidget(new Core::FindToolBarPlaceHolder(this));
-#endif
     setLayout(layout);
 
-#ifndef GDBDEBUGGERLEAN
     Aggregation::Aggregate *aggregate = new Aggregation::Aggregate;
     aggregate->add(m_combinedText);
     aggregate->add(new BaseTextFind(m_combinedText));
@@ -329,7 +366,6 @@ DebuggerOutputWindow::DebuggerOutputWindow(QWidget *parent)
     aggregate = new Aggregation::Aggregate;
     aggregate->add(m_inputText);
     aggregate->add(new BaseTextFind(m_inputText));
-#endif
 
     connect(m_inputText, SIGNAL(statusMessageRequested(QString,int)),
        this, SIGNAL(statusMessageRequested(QString,int)));
@@ -341,18 +377,25 @@ void DebuggerOutputWindow::showOutput(int channel, const QString &output)
 {
     if (output.isEmpty())
         return;
+    QTextCursor oldCursor = m_combinedText->textCursor();
+    QTextCursor cursor = oldCursor;
+    cursor.movePosition(QTextCursor::End);
+    bool atEnd = oldCursor.position() == cursor.position();
     foreach (QString line, output.split('\n')) {
         // FIXME: QTextEdit asserts on really long lines...
         const int n = 30000;
-        if (line.size() > n)
-            line = line.left(n) + " [...] <cut off>";
+        if (line.size() > n) {
+            line.truncate(n);
+            line += QLatin1String(" [...] <cut off>");
+        }
         if (line != QLatin1String("(gdb) "))
             m_combinedText->appendPlainText(charForChannel(channel) + line);
     }
-    QTextCursor cursor = m_combinedText->textCursor();
     cursor.movePosition(QTextCursor::End);
-    m_combinedText->setTextCursor(cursor);
-    m_combinedText->ensureCursorVisible();
+    if (atEnd) {
+        m_combinedText->setTextCursor(cursor);
+        m_combinedText->ensureCursorVisible();
+    }
 }
 
 void DebuggerOutputWindow::showInput(int channel, const QString &input)
