@@ -2,7 +2,7 @@
 **
 ** This file is part of Qt Creator
 **
-** Copyright (c) 2009 Nokia Corporation and/or its subsidiary(-ies).
+** Copyright (c) 2010 Nokia Corporation and/or its subsidiary(-ies).
 **
 ** Contact: Nokia Corporation (qt-info@nokia.com)
 **
@@ -36,8 +36,10 @@
 #include "watchutils.h"
 
 #include <QtCore/QByteArray>
+#include <QtCore/QFutureInterface>
 #include <QtCore/QHash>
 #include <QtCore/QMap>
+#include <QtCore/QMultiMap>
 #include <QtCore/QObject>
 #include <QtCore/QProcess>
 #include <QtCore/QPoint>
@@ -45,14 +47,12 @@
 #include <QtCore/QTextCodec>
 #include <QtCore/QTime>
 #include <QtCore/QVariant>
+#include <QtCore/QSharedPointer>
 
 QT_BEGIN_NAMESPACE
-class QAbstractItemModel;
-class QAction;
 class QMainWindow;
 class QMessageBox;
 class QTimer;
-class QWidget;
 QT_END_NAMESPACE
 
 namespace Debugger {
@@ -65,13 +65,13 @@ class GdbMi;
 
 class BreakpointData;
 class WatchData;
+class DisassemblerAgentCookie;
 
 class AttachGdbAdapter;
 class CoreGdbAdapter;
 class PlainGdbAdapter;
 class RemoteGdbAdapter;
 class TrkGdbAdapter;
-struct TrkOptions;
 
 enum DebuggingHelperState
 {
@@ -81,11 +81,15 @@ enum DebuggingHelperState
     DebuggingHelperUnavailable,
 };
 
+
 class GdbEngine : public IDebuggerEngine
 {
     Q_OBJECT
 
 public:
+    typedef QMultiMap<QString, int> GdbBinaryToolChainMap;
+    typedef QSharedPointer<GdbBinaryToolChainMap> GdbBinaryToolChainMapPtr;
+
     explicit GdbEngine(DebuggerManager *manager);
     ~GdbEngine();
 
@@ -102,16 +106,17 @@ private: ////////// General Interface //////////
 
     virtual void addOptionPages(QList<Core::IOptionsPage*> *opts) const;
 
-    virtual bool checkConfiguration(int toolChain, QString *errorMessage, QString *settingsPage= 0) const;
-
-    virtual bool isGdbEngine() const { return true; }
-
+    virtual bool checkConfiguration(int toolChain, QString *errorMessage,
+        QString *settingsPage = 0) const;
     virtual void startDebugger(const DebuggerStartParametersPtr &sp);
+    virtual unsigned debuggerCapabilities() const;
     virtual void exitDebugger();
+    virtual void abortDebugger();
     virtual void detachDebugger();
     virtual void shutdown();
 
     virtual void executeDebuggerCommand(const QString &command);
+    virtual QString qtNamespace() const { return m_dumperHelper.qtNamespace(); }
 
 private: ////////// General State //////////
 
@@ -122,7 +127,6 @@ private: ////////// General State //////////
     Q_SLOT void setAutoDerefPointers(const QVariant &on);
 
     DebuggerStartParametersPtr m_startParameters;
-    QSharedPointer<TrkOptions> m_trkOptions;
     bool m_registerNamesListed;
 
 private: ////////// Gdb Process Management //////////
@@ -137,11 +141,6 @@ private: ////////// Gdb Process Management //////////
     void handleInferiorShutdown(const GdbResponse &response);
     void handleGdbExit(const GdbResponse &response);
 
-    void gdbInputAvailable(int channel, const QString &msg)
-    { m_manager->showDebuggerInput(channel, msg); }
-    void gdbOutputAvailable(int channel, const QString &msg)
-    { m_manager->showDebuggerOutput(channel, msg); }
-
 private slots:
     void handleGdbFinished(int, QProcess::ExitStatus status);
     void handleGdbError(QProcess::ProcessError error);
@@ -150,7 +149,8 @@ private slots:
     void readDebugeeOutput(const QByteArray &data);
 
     void handleAdapterStarted();
-    void handleAdapterStartFailed(const QString &msg, const QString &settingsIdHint = QString());
+    void handleAdapterStartFailed(const QString &msg,
+        const QString &settingsIdHint = QString());
 
     void handleInferiorPrepared();
 
@@ -170,18 +170,28 @@ private:
 
 private: ////////// Gdb Command Management //////////
 
-    public: // otherwise the Qt flag macros are unhappy
+    public: // Otherwise the Qt flag macros are unhappy.
     enum GdbCommandFlag {
         NoFlags = 0,
-        NeedsStop = 1,    // The command needs a stopped inferior
-        Discardable = 2,  // No need to wait for the reply before continuing inferior
-        RebuildModel = 4, // Trigger model rebuild when no such commands are pending any more
-        WatchUpdate = Discardable | RebuildModel,
-        NonCriticalResponse = 8,  // We can live without recieving an answer
-        RunRequest = 16,  // Callback expects GdbResultRunning instead of GdbResultDone
-        ExitRequest = 32, // Callback expects GdbResultExit instead of GdbResultDone
-        LosesChild = 64,   // Auto-set inferior shutdown related states
-        EmbedToken = 128   // Expand %1 in the command to the command token
+        // The command needs a stopped inferior.
+        NeedsStop = 1, 
+        // No need to wait for the reply before continuing inferior.
+        Discardable = 2,
+        // Trigger watch model rebuild when no such commands are pending anymore.
+        RebuildWatchModel = 4,
+        WatchUpdate = Discardable | RebuildWatchModel,
+        // We can live without recieving an answer.
+        NonCriticalResponse = 8,
+        // Callback expects GdbResultRunning instead of GdbResultDone.
+        RunRequest = 16,
+        // Callback expects GdbResultExit instead of GdbResultDone.
+        ExitRequest = 32,
+        // Auto-set inferior shutdown related states.
+        LosesChild = 64,
+        // Trigger breakpoint model rebuild when no such commands are pending anymore.
+        RebuildBreakpointModel = 128,
+        // This command needs to be send immediately.
+        Immediate = 256,
     };
     Q_DECLARE_FLAGS(GdbCommandFlags, GdbCommandFlag)
     private:
@@ -201,30 +211,30 @@ private: ////////// Gdb Command Management //////////
         GdbCommandCallback callback;
         AdapterCallback adapterCallback;
         const char *callbackName;
-        QString command;
+        QByteArray command;
         QVariant cookie;
         QTime postTime;
     };
 
-    // type and cookie are sender-internal data, opaque for the "event
+    // Type and cookie are sender-internal data, opaque for the "event
     // queue". resultNeeded == true increments m_pendingResults on
     // send and decrements on receipt, effectively preventing
     // watch model updates before everything is finished.
     void flushCommand(const GdbCommand &cmd);
-    void postCommand(const QString &command,
+    void postCommand(const QByteArray &command,
                      GdbCommandFlags flags,
                      GdbCommandCallback callback = 0,
                      const char *callbackName = 0,
                      const QVariant &cookie = QVariant());
-    void postCommand(const QString &command,
+    void postCommand(const QByteArray &command,
                      GdbCommandCallback callback = 0,
                      const char *callbackName = 0,
                      const QVariant &cookie = QVariant());
-    void postCommand(const QString &command,
+    void postCommand(const QByteArray &command,
                      AdapterCallback callback,
                      const char *callbackName,
                      const QVariant &cookie = QVariant());
-    void postCommand(const QString &command,
+    void postCommand(const QByteArray &command,
                      GdbCommandFlags flags,
                      AdapterCallback callback,
                      const char *callbackName,
@@ -241,18 +251,20 @@ private: ////////// Gdb Command Management //////////
     QByteArray m_pendingConsoleStreamOutput;
     QByteArray m_pendingLogStreamOutput;
 
-    // contains the first token number for the current round
+    // This contains the first token number for the current round
     // of evaluation. Responses with older tokens are considers
     // out of date and discarded.
     int m_oldestAcceptableToken;
 
-    int m_pendingRequests; // Watch updating commands in flight
+    int m_pendingWatchRequests; // Watch updating commands in flight
+    int m_pendingBreakpointRequests; // Watch updating commands in flight
 
     typedef void (GdbEngine::*CommandsDoneCallback)();
-    // function called after all previous responses have been received
+    // This function is called after all previous responses have been received.
     CommandsDoneCallback m_commandsDoneCallback;
 
     QList<GdbCommand> m_commandsToRunOnTemporaryBreak;
+    int gdbVersion() const { return m_gdbVersion; }
 
 private: ////////// Gdb Output, State & Capability Handling //////////
 
@@ -263,28 +275,29 @@ private: ////////// Gdb Output, State & Capability Handling //////////
     void handleStop1(const GdbMi &data);
     StackFrame parseStackFrame(const GdbMi &mi, int level);
 
-    virtual bool isSynchroneous() const;
+    bool isSynchroneous() const { return hasPython(); }
+    virtual bool hasPython() const;
     bool supportsThreads() const;
 
     // Gdb initialization sequence
     void handleShowVersion(const GdbResponse &response);
-    void handleIsSynchroneous(const GdbResponse &response);
+    void handleHasPython(const GdbResponse &response);
 
     int m_gdbVersion; // 6.8.0 is 60800
     int m_gdbBuildVersion; // MAC only?
     bool m_isMacGdb;
-    bool m_isSynchroneous; // Can act synchroneously?
+    bool m_hasPython;
 
 private: ////////// Inferior Management //////////
 
     // This should be always the last call in a function.
     Q_SLOT virtual void attemptBreakpointSynchronization();
 
-    virtual void stepExec();
-    virtual void stepOutExec();
-    virtual void nextExec();
-    virtual void stepIExec();
-    virtual void nextIExec();
+    virtual void executeStep();
+    virtual void executeStepOut();
+    virtual void executeNext();
+    virtual void executeStepI();
+    virtual void executeNextI();
 
     void continueInferiorInternal();
     void autoContinueInferior();
@@ -292,12 +305,16 @@ private: ////////// Inferior Management //////////
     virtual void interruptInferior();
     void interruptInferiorTemporarily();
 
-    virtual void runToLineExec(const QString &fileName, int lineNumber);
-    virtual void runToFunctionExec(const QString &functionName);
+    virtual void executeRunToLine(const QString &fileName, int lineNumber);
+    virtual void executeRunToFunction(const QString &functionName);
 //    void handleExecRunToFunction(const GdbResponse &response);
-    virtual void jumpToLineExec(const QString &fileName, int lineNumber);
+    virtual void executeJumpToLine(const QString &fileName, int lineNumber);
+    virtual void executeReturn();
 
-    void handleExecContinue(const GdbResponse &response);
+    void handleExecuteContinue(const GdbResponse &response);
+    void handleExecuteStep(const GdbResponse &response);
+    void handleExecuteNext(const GdbResponse &response);
+    void handleExecuteReturn(const GdbResponse &response);
 
     qint64 inferiorPid() const { return m_manager->inferiorPid(); }
     void handleInferiorPidChanged(qint64 pid) { manager()->notifyInferiorPidChanged(pid); }
@@ -308,6 +325,7 @@ private: ////////// Inferior Management //////////
 
     QByteArray m_entryPoint;
 #endif
+    QFutureInterface<void> *m_progress;
 
 private: ////////// View & Data Stuff //////////
 
@@ -322,12 +340,15 @@ private: ////////// View & Data Stuff //////////
     void handleBreakList(const GdbResponse &response);
     void handleBreakList(const GdbMi &table);
     void handleBreakIgnore(const GdbResponse &response);
-    void handleBreakInsert(const GdbResponse &response);
+    void handleBreakDisable(const GdbResponse &response);
     void handleBreakInsert1(const GdbResponse &response);
+    void handleBreakInsert2(const GdbResponse &response);
     void handleBreakCondition(const GdbResponse &response);
     void handleBreakInfo(const GdbResponse &response);
+    void handleInfoLine(const GdbResponse &response);
     void extractDataFromInfoBreak(const QString &output, BreakpointData *data);
     void breakpointDataFromOutput(BreakpointData *data, const GdbMi &bkpt);
+    QByteArray breakpointLocation(int index);
     void sendInsertBreakpoint(int index);
     QString breakLocation(const QString &file) const;
     void reloadBreakListInternal();
@@ -344,6 +365,15 @@ private: ////////// View & Data Stuff //////////
 
     bool m_modulesListOutdated;
 
+    //
+    // Snapshot specific stuff
+    //
+    virtual void makeSnapshot();
+    void handleMakeSnapshot(const GdbResponse &response);
+    void handleActivateSnapshot(const GdbResponse &response);
+    void activateSnapshot(int index);
+    void activateSnapshot2();
+
     //
     // Register specific stuff
     //
@@ -355,10 +385,13 @@ private: ////////// View & Data Stuff //////////
     //
     // Disassembler specific stuff
     //
-    virtual void fetchDisassembler(DisassemblerViewAgent *agent,
-        const StackFrame &frame);
-    void fetchDisassemblerByAddress(DisassemblerViewAgent *agent,
+    virtual void fetchDisassembler(DisassemblerViewAgent *agent);
+    void fetchDisassemblerByAddress(const DisassemblerAgentCookie &ac,
         bool useMixedMode);
+    void fetchDisassemblerByCli(const DisassemblerAgentCookie &ac,
+        bool useMixedMode);
+    void fetchDisassemblerByAddressCli(const DisassemblerAgentCookie &ac);
+    void handleFetchDisassemblerByCli(const GdbResponse &response);
     void handleFetchDisassemblerByLine(const GdbResponse &response);
     void handleFetchDisassemblerByAddress1(const GdbResponse &response);
     void handleFetchDisassemblerByAddress0(const GdbResponse &response);
@@ -386,16 +419,17 @@ private: ////////// View & Data Stuff //////////
     bool m_sourcesListOutdated;
     bool m_sourcesListUpdating;
     bool m_breakListOutdated;
-    bool m_breakListUpdating;
 
     //
     // Stack specific stuff
     //
     void updateAll();
+        void updateAllClassic();
+        void updateAllPython();
     void handleStackListFrames(const GdbResponse &response);
     void handleStackSelectThread(const GdbResponse &response);
+    void handleStackSelectFrame(const GdbResponse &response);
     void handleStackListThreads(const GdbResponse &response);
-    void handleStackFrame(const GdbResponse &response);
     Q_SLOT void reloadStack(bool forceGotoLocation);
     Q_SLOT virtual void reloadFullStack();
     int currentFrame() const;
@@ -406,75 +440,82 @@ private: ////////// View & Data Stuff //////////
     //
     // Watch specific stuff
     //
-    virtual void setToolTipExpression(const QPoint &mousePos, TextEditor::ITextEditor *editor, int cursorPos);
+    virtual void setToolTipExpression(const QPoint &mousePos,
+        TextEditor::ITextEditor *editor, int cursorPos);
 
     virtual void assignValueInDebugger(const QString &expr, const QString &value);
 
-    virtual void fetchMemory(MemoryViewAgent *agent, quint64 addr, quint64 length);
+    virtual void fetchMemory(MemoryViewAgent *agent, QObject *token,
+        quint64 addr, quint64 length);
     void handleFetchMemory(const GdbResponse &response);
 
     virtual void watchPoint(const QPoint &);
     void handleWatchPoint(const GdbResponse &response);
 
     // FIXME: BaseClass. called to improve situation for a watch item
-    void updateSubItem(const WatchData &data);
-    void handleChildren(const WatchData &parent, const GdbMi &child,
-        QList<WatchData> *insertions);
+    void updateSubItemClassic(const WatchData &data);
 
     void virtual updateWatchData(const WatchData &data);
     Q_SLOT void updateWatchDataHelper(const WatchData &data);
-    void rebuildModel();
+    void rebuildWatchModel();
     bool showToolTip();
 
     void insertData(const WatchData &data);
     void sendWatchParameters(const QByteArray &params0);
-    void createGdbVariable(const WatchData &data);
+    void createGdbVariableClassic(const WatchData &data);
 
-    void runDebuggingHelper(const WatchData &data, bool dumpChildren);
-    void runDirectDebuggingHelper(const WatchData &data, bool dumpChildren);
+    void runDebuggingHelperClassic(const WatchData &data, bool dumpChildren);
+    void runDirectDebuggingHelperClassic(const WatchData &data, bool dumpChildren);
     bool hasDebuggingHelperForType(const QString &type) const;
 
-    void handleVarListChildren(const GdbResponse &response);
-    void handleVarListChildrenHelper(const GdbMi &child,
+    void handleVarListChildrenClassic(const GdbResponse &response);
+    void handleVarListChildrenHelperClassic(const GdbMi &child,
         const WatchData &parent);
     void handleVarCreate(const GdbResponse &response);
     void handleVarAssign(const GdbResponse &response);
-    void handleEvaluateExpression(const GdbResponse &response);
-    //void handleToolTip(const GdbResponse &response);
-    void handleQueryDebuggingHelper(const GdbResponse &response);
-    void handleDebuggingHelperValue2(const GdbResponse &response);
-    void handleDebuggingHelperValue3(const GdbResponse &response);
+    void handleEvaluateExpressionClassic(const GdbResponse &response);
+    void handleQueryDebuggingHelperClassic(const GdbResponse &response);
+    void handleDebuggingHelperValue2Classic(const GdbResponse &response);
+    void handleDebuggingHelperValue3Classic(const GdbResponse &response);
     void handleDebuggingHelperEditValue(const GdbResponse &response);
     void handleDebuggingHelperSetup(const GdbResponse &response);
 
+    Q_SLOT void createFullBacktrace();
+    void handleCreateFullBacktrace(const GdbResponse &response);
+
     void updateLocals(const QVariant &cookie = QVariant());
-    void handleStackListLocals(const GdbResponse &response);
+        void updateLocalsClassic(const QVariant &cookie);
+        void updateLocalsPython(const QByteArray &varList);
+            void handleStackFramePython(const GdbResponse &response);
+
+    void handleStackListLocalsClassic(const GdbResponse &response);
+    void handleStackListLocalsPython(const GdbResponse &response);
+
     WatchData localVariable(const GdbMi &item,
                             const QStringList &uninitializedVariables,
                             QMap<QByteArray, int> *seen);
     void setLocals(const QList<GdbMi> &locals);
-    void handleStackListArguments(const GdbResponse &response);
-    void setWatchDataType(WatchData &data, const GdbMi &mi);
-    void setWatchDataDisplayedType(WatchData &data, const GdbMi &mi);
+    void handleStackListArgumentsClassic(const GdbResponse &response);
 
-    QSet<QString> m_processedNames;
-    QMap<QString, QString> m_varToType;
+    QSet<QByteArray> m_processedNames;
 
 private: ////////// Dumper Management //////////
     QString qtDumperLibraryName() const;
     bool checkDebuggingHelpers();
-    void setDebuggingHelperState(DebuggingHelperState);
-    void tryLoadDebuggingHelpers();
-    void tryQueryDebuggingHelpers();
-    Q_SLOT void recheckDebuggingHelperAvailability();
+        bool checkDebuggingHelpersClassic();
+    void setDebuggingHelperStateClassic(DebuggingHelperState);
+    void tryLoadDebuggingHelpersClassic();
+    void tryQueryDebuggingHelpersClassic();
+    Q_SLOT void recheckDebuggingHelperAvailabilityClassic();
     void connectDebuggingHelperActions();
     void disconnectDebuggingHelperActions();
-    Q_SLOT void setDebugDebuggingHelpers(const QVariant &on);
+    Q_SLOT void setDebugDebuggingHelpersClassic(const QVariant &on);
     Q_SLOT void setUseDebuggingHelpers(const QVariant &on);
 
-    const bool m_dumperInjectionLoad;
+    const GdbBinaryToolChainMapPtr m_gdbBinaryToolChainMap;
     DebuggingHelperState m_debuggingHelperState;
     QtDumperHelper m_dumperHelper;
+    QString m_gdb;
 
 private: ////////// Convenience Functions //////////
 
@@ -483,6 +524,10 @@ private: ////////// Convenience Functions //////////
         int buttons = 0);
     void debugMessage(const QString &msg);
     QMainWindow *mainWindow() const;
+
+    static QString m_toolTipExpression;
+    static QPoint m_toolTipPos;
+    static QByteArray tooltipINameForExpression(const QByteArray &exp);
 };
 
 } // namespace Internal
